From 4438a15635cc534a481ba3ca0ed083b1729bee13 Mon Sep 17 00:00:00 2001
From: Marcus Asteborg <maastebo@microsoft.com>
Date: Tue, 17 Dec 2019 09:07:11 -0800
Subject: [PATCH 5/5] CMake SSE fixes, only compile SSE source with SSE flags
 on non-windows when target use runtime check #154. For windows we only use
 /arch flag when target is presumed to support SSE #132. Added support for
 enable float approx option"

---
 CMakeLists.txt       | 127 ++++++++++++++++++++++++++++++++++---------
 opus_functions.cmake |  48 ++++++++--------
 2 files changed, 127 insertions(+), 48 deletions(-)

diff --git a/CMakeLists.txt b/CMakeLists.txt
index ba86939a..df876326 100644
--- a/CMakeLists.txt
+++ b/CMakeLists.txt
@@ -20,7 +20,11 @@ include(opus_buildtype.cmake)
 
 option(OPUS_BUILD_SHARED_LIBRARY "Build shared library" OFF)
 option(OPUS_STACK_PROTECTOR "Use stack protection" ON)
-option(OPUS_USE_ALLOCA "Use alloca for stack arrays (on non-C99 compilers)" OFF)
+if (MSVC)
+   option(OPUS_USE_ALLOCA "Use alloca for stack arrays (on non-C99 compilers)" ON)
+else()
+   option(OPUS_USE_ALLOCA "Use alloca for stack arrays (on non-C99 compilers)" OFF)
+endif()
 option(OPUS_CUSTOM_MODES "Enable non-Opus modes, e.g. 44.1 kHz & 2^n frames"
        OFF)
 option(OPUS_BUILD_PROGRAMS "Build programs" OFF)
@@ -29,6 +33,9 @@ option(OPUS_FIXED_POINT
 option(OPUS_ENABLE_FLOAT_API
        "Compile with the floating point API (for machines with float library"
        ON)
+option(OPUS_ENABLE_FLOAT_APPROX
+       "Enable floating point approximations"
+       OFF)
 option(OPUS_INSTALL_PKG_CONFIG_MODULE "Install PkgConfig module" ON)
 option(OPUS_INSTALL_CMAKE_CONFIG_MODULE "Install CMake package config module"
        ON)
@@ -55,6 +62,15 @@ else()
   endif()
 endif()
 
+if(OPUS_ENABLE_FLOAT_APPROX)
+  add_compile_definitions(FLOAT_APPROX)
+  if(MSVC)
+    check_and_set_flag(ENABLE_FLOAT_APPROX /fp:fast)
+  else()
+    check_and_set_flag(ENABLE_FLOAT_APPROX -ffast-math)
+  endif()
+endif()
+
 if(OPUS_CPU_X86 OR OPUS_CPU_X64)
   cmake_dependent_option(OPUS_X86_MAY_HAVE_SSE
                          "Does runtime check for SSE1 support"
@@ -136,6 +152,9 @@ add_feature_info(
 add_feature_info(
   FLOAT_API OPUS_ENABLE_FLOAT_API
   "compile with the floating point API (for machines with float library)")
+add_feature_info(
+  ENABLE_FLOAT_APPROX OPUS_ENABLE_FLOAT_APPROX
+   "Enable floating point approximations")
 
 add_feature_info(INSTALL_PKG_CONFIG_MODULE OPUS_INSTALL_PKG_CONFIG_MODULE
                  "install PkgConfig module")
@@ -203,7 +222,7 @@ endif()
 # variable-length arrays for stack allocation USE_ALLOCA: Use alloca() for stack
 # allocation If none is defined, then the fallback is a non-threadsafe global
 # array
-if(OPUS_USE_ALLOCA OR MSVC)
+if(OPUS_USE_ALLOCA)
   target_compile_definitions(opus PRIVATE USE_ALLOCA)
 else()
   target_compile_definitions(opus PRIVATE VAR_ARRAYS)
@@ -241,39 +260,97 @@ if(NOT OPUS_ENABLE_FLOAT_API)
   target_compile_definitions(opus PRIVATE DISABLE_FLOAT_API)
 endif()
 
-if(OPUS_X86_MAY_HAVE_SSE
-   OR OPUS_X86_MAY_HAVE_SSE2
-   OR OPUS_X86_MAY_HAVE_SSE4_1
-   OR OPUS_X86_MAY_HAVE_AVX)
+#[[Build flags for SSE will be set the following way:
+MSVC: If OPUS_X86_PRESUME_X is set then we will set the highest possible /arch:X
+we won't set any ARCH flag for OPUS_X86_MAY_HAVE_SSE due to:
+https://randomascii.wordpress.com/2016/12/05/vc-archavx-option-unsafe-at-any-speed/
+For non MSVC: we will set the compiler flags on per file basis for OPUS_X86_MAY_HAVE_SSE
+for OPUS_X86_PRESUME_X we will set it for the target]]
+
+if((OPUS_X86_MAY_HAVE_SSE AND NOT OPUS_X86_PRESUME_SSE) OR
+   (OPUS_X86_MAY_HAVE_SSE2 AND NOT OPUS_X86_PRESUME_SSE2) OR
+   (OPUS_X86_MAY_HAVE_SSE4_1 AND NOT OPUS_X86_PRESUME_SSE4_1) OR
+   (OPUS_X86_MAY_HAVE_AVX AND NOT OPUS_X86_PRESUME_AVX))
   target_compile_definitions(opus PRIVATE OPUS_HAVE_RTCD)
 endif()
 
-if(OPUS_X86_MAY_HAVE_SSE)
-  add_sources_group(opus celt ${celt_sources_sse})
-  target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE)
-endif()
-if(OPUS_X86_PRESUME_SSE)
-  target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE)
+if(SSE1_SUPPORTED)
+  if(OPUS_X86_MAY_HAVE_SSE)
+    add_sources_group(opus celt ${celt_sources_sse})
+    target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE)
+    if(NOT MSVC)
+      set_source_files_properties(${celt_sources_sse} PROPERTIES COMPILE_FLAGS -msse)
+    endif()
+  endif()
+  if(OPUS_X86_PRESUME_SSE)
+    target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE)
+    if(NOT MSVC)
+      target_compile_options(opus PRIVATE -msse)
+    endif()
+  endif()
 endif()
 
-if(OPUS_X86_MAY_HAVE_SSE2)
-  add_sources_group(opus celt ${celt_sources_sse2})
-  target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE2)
+if(SSE2_SUPPORTED)
+  if(OPUS_X86_MAY_HAVE_SSE2)
+    add_sources_group(opus celt ${celt_sources_sse2})
+    target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE2)
+    if(NOT MSVC)
+      set_source_files_properties(${celt_sources_sse2} PROPERTIES COMPILE_FLAGS -msse2)
+    endif()
+  endif()
+  if(OPUS_X86_PRESUME_SSE2)
+    target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE2)
+    if(NOT MSVC)
+      target_compile_options(opus PRIVATE -msse2)
+    endif()
+  endif()
 endif()
-if(OPUS_X86_PRESUME_SSE2)
-  target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE2)
+
+if(SSE4_1_SUPPORTED)
+  if(OPUS_X86_MAY_HAVE_SSE4_1)
+    add_sources_group(opus celt ${celt_sources_sse4_1})
+    add_sources_group(opus silk ${silk_sources_sse4_1})
+    target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE4_1)
+    if(NOT MSVC)
+      set_source_files_properties(${celt_sources_sse4_1} ${silk_sources_sse4_1} PROPERTIES COMPILE_FLAGS -msse4.1)
+    endif()
+
+    if(OPUS_FIXED_POINT)
+      add_sources_group(opus silk ${silk_sources_fixed_sse4_1})
+      if(NOT MSVC)
+        set_source_files_properties(${silk_sources_fixed_sse4_1} PROPERTIES COMPILE_FLAGS -msse4.1)
+      endif()
+    endif()
+  endif()
+  if(OPUS_X86_PRESUME_SSE4_1)
+    target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE4_1)
+    if(NOT MSVC)
+      target_compile_options(opus PRIVATE -msse4.1)
+    endif()
+  endif()
 endif()
 
-if(OPUS_X86_MAY_HAVE_SSE)
-  add_sources_group(opus celt ${celt_sources_sse4_1})
-  add_sources_group(opus silk ${silk_sources_sse4_1})
-  if(OPUS_FIXED_POINT)
-    add_sources_group(opus silk ${silk_sources_fixed_sse4_1})
+if(AVX_SUPPORTED)
+  target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_AVX)
+  if(OPUS_X86_PRESUME_AVX)
+    target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_AVX)
+    if(NOT MSVC)
+      target_compile_options(opus PRIVATE -mavx)
+    endif()
   endif()
-  target_compile_definitions(opus PRIVATE OPUS_X86_MAY_HAVE_SSE4_1)
 endif()
-if(OPUS_X86_PRESUME_SSE4_1)
-  target_compile_definitions(opus PRIVATE OPUS_X86_PRESUME_SSE4_1)
+
+if(MSVC) # To avoid warning D9025 of overriding compiler options
+  if(AVX_SUPPORTED AND OPUS_X86_PRESUME_AVX) # on 64 bit and 32 bits
+    add_definitions(/arch:AVX)
+  elseif(OPUS_CPU_X86) # if AVX not supported then set SSE flag
+    if((SSE4_1_SUPPORTED AND OPUS_X86_PRESUME_SSE4_1)
+       OR (SSE2_SUPPORTED AND OPUS_X86_PRESUME_SSE2))
+      target_compile_definitions(opus PRIVATE /arch:SSE2)
+    elseif(SSE1_SUPPORTED AND OPUS_X86_PRESUME_SSE)
+      target_compile_definitions(opus PRIVATE /arch:SSE)
+    endif()
+  endif()
 endif()
 
 if(CMAKE_SYSTEM_PROCESSOR MATCHES "(armv7-a)")
diff --git a/opus_functions.cmake b/opus_functions.cmake
index fe309c2c..2692bfce 100644
--- a/opus_functions.cmake
+++ b/opus_functions.cmake
@@ -115,13 +115,17 @@ function(opus_detect_sse COMPILER_SUPPORT_SIMD)
       if(CMAKE_SIZEOF_VOID_P EQUAL 4)
         check_flag(SSE1 /arch:SSE)
       else()
-        set(SSE1_SUPPORTED 1 PARENT_SCOPE)
+        set(SSE1_SUPPORTED
+            1
+            PARENT_SCOPE)
       endif()
     else()
-      check_and_set_flag(SSE1 -msse)
+      check_flag(SSE1 -msse)
     endif()
   else()
-    set(SSE1_SUPPORTED 0 PARENT_SCOPE)
+    set(SSE1_SUPPORTED
+        0
+        PARENT_SCOPE)
   endif()
 
   check_include_file(emmintrin.h HAVE_EMMINTRIN_H) # SSE2
@@ -130,13 +134,17 @@ function(opus_detect_sse COMPILER_SUPPORT_SIMD)
       if(CMAKE_SIZEOF_VOID_P EQUAL 4)
         check_flag(SSE2 /arch:SSE2)
       else()
-        set(SSE2_SUPPORTED 1 PARENT_SCOPE)
+        set(SSE2_SUPPORTED
+            1
+            PARENT_SCOPE)
       endif()
     else()
-      check_and_set_flag(SSE2 -msse2)
+      check_flag(SSE2 -msse2)
     endif()
   else()
-    set(SSE2_SUPPORTED 0 PARENT_SCOPE)
+    set(SSE2_SUPPORTED
+        0
+        PARENT_SCOPE)
   endif()
 
   check_include_file(smmintrin.h HAVE_SMMINTRIN_H) # SSE4.1
@@ -145,13 +153,17 @@ function(opus_detect_sse COMPILER_SUPPORT_SIMD)
       if(CMAKE_SIZEOF_VOID_P EQUAL 4)
         check_flag(SSE4_1 /arch:SSE2) # SSE2 and above
       else()
-        set(SSE4_1_SUPPORTED 1 PARENT_SCOPE)
+        set(SSE4_1_SUPPORTED
+            1
+            PARENT_SCOPE)
       endif()
     else()
-      check_and_set_flag(SSE4_1 -msse4.1)
+      check_flag(SSE4_1 -msse4.1)
     endif()
   else()
-    set(SSE4_1_SUPPORTED 0 PARENT_SCOPE)
+    set(SSE4_1_SUPPORTED
+        0
+        PARENT_SCOPE)
   endif()
 
   check_include_file(immintrin.h HAVE_IMMINTRIN_H) # AVX
@@ -159,22 +171,12 @@ function(opus_detect_sse COMPILER_SUPPORT_SIMD)
     if(MSVC)
       check_flag(AVX /arch:AVX)
     else()
-      check_and_set_flag(AVX -mavx)
+      check_flag(AVX -mavx)
     endif()
   else()
-    set(AVX_SUPPORTED 0 PARENT_SCOPE)
-  endif()
-
-  if(MSVC) # To avoid warning D9025 of overriding compiler options
-    if(AVX_SUPPORTED) # on 64 bit and 32 bits
-      add_definitions(/arch:AVX)
-    elseif(CMAKE_SIZEOF_VOID_P EQUAL 4) # if AVX not supported then set SSE flag
-      if(SSE4_1_SUPPORTED OR SSE2_SUPPORTED)
-        add_definitions(/arch:SSE2)
-      elseif(SSE1_SUPPORTED)
-        add_definitions(/arch:SSE)
-      endif()
-    endif()
+    set(AVX_SUPPORTED
+        0
+        PARENT_SCOPE)
   endif()
 
   if(SSE1_SUPPORTED OR SSE2_SUPPORTED OR SSE4_1_SUPPORTED OR AVX_SUPPORTED)
-- 
2.23.0.windows.1

